import os
import sys

import libpythonc.support
from libpythonc.support import argparse
from libpythonc.support import logging


class MyFormatter(argparse.ArgumentDefaultsHelpFormatter):
    def __init__(self, *args, **kwargs):
        if len(args) != 4 and not 'width' in kwargs:
            if not os.getenv("COLUMNS", None):
                print >>sys.stderr, "no COLUMNS defined in the environment"
            kwargs['width'] = int(os.getenv("COLUMNS", 100000))

        super(MyFormatter, self).__init__(*args, **kwargs)


class StandardParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        if not 'formatter_class' in kwargs:
            kwargs['formatter_class'] = MyFormatter

        super(StandardParser, self).__init__(*args, **kwargs)

        group = self.add_mutually_exclusive_group()
        group.add_argument("-v", "--verbose", action="store_true", default=False, help="increases logging verbosity")
        group.add_argument("-q", "--quiet", action="store_true", default=False, help="reduces logging verbosity")

        self.add_argument("--log", metavar="<file>", default=None, help="output log file")
        self.add_argument("--log-format", default=logging.BASIC_FORMAT, help="record's log format")
        self.add_argument("--pdb", action="store_true", default=False, help="on error launch the pdb debugger")


    def parse_args(self, *args, **kwargs):
        options = super(StandardParser, self).parse_args(*args, **kwargs)
        
        logging.basicConfig()
        logging.getLogger().setLevel(logging.INFO)
        if options.quiet == True:
            logging.getLogger().setLevel(logging.WARN)
        if options.verbose == True:
            logging.getLogger().setLevel(logging.DEBUG)

        if options.pdb == True:
            logging.warn("catching un-handled exception with pdb")
            sys.excepthook = lambda a, b, c: pdb.post_mortem(c)

        if options.log != None:
            logging.getLogger().handlers = [ logging.FileHandler(options.log, mode="w"), ]
        
        for h in logging.getLogger().handlers:
            h.setFormatter(logging.Formatter(fmt=options.log_format))

        return options
