#!/usr/bin/env python
import sys
import os
import shutil

def npath(*args):
    "joins *args into an absolute pathname -> string"
    return os.path.normpath(os.path.abspath(os.path.expanduser(os.path.join(*args))))

# This configure script can be found inside autolib (so we use with a fullpath call)
# or it can copied into a project (normally one level above autolib dir)

try:
    import autolib
except ImportError:
    sys.path.insert(0, npath(__file__, "..", ".."))
    try:
        import autolib
    except ImportError:
        del sys.path[0]
        sys.path.insert(0, npath(__file__, ".."))
        import autolib


from autolib import tools, logging, platform

log = logging.getLogger("configure")


def parse_args(args):
    parser = support.CliParser()

    default = os.getenv("BUILD", npath("build"))
    parser.add_argument("--build",     metavar="$BUILD", default=default, help="writeable build dir")

    default = os.getenv("DOWNLOADS", npath("downloads"))
    parser.add_argument("--download-dir", metavar="$DOWNLOADS", default=default, help="downloaded files dir")

    default = os.getenv("PREFIX", npath("~", ".local", "packages"))
    parser.add_argument("--prefix",       metavar="$PREFIX", default=default, help="prefix install")

    options = parser.parse_args(args)

    options.build = npath(options.build)
    options.downloads = npath(options.download_dir)
    options.prefix = npath(options.prefix)
    return options


def populate(build, scriptdir):
    
    scripts = os.listdir(scriptdir)
    for script in os.listdir(scriptdir):
        src = npath(scriptdir, script)
        dst = npath(build, script)
        log.debug("copying '%s' -> '%s'" % (src, dst))
        shutil.copyfile(src, dst)
        os.chmod(dst, 0755)


def configure(basedir, build, downloads, prefix):
    log = logging.getLogger("configure")

    log.info("set basedir('%s')" % basedir)
    log.info("set downloads('%s')" % downloads)
    log.info("set build('%s')" % build)
    log.info("set prefix('%s')" % prefix)


    # creating the build root
    if not os.path.exists(build):
        os.makedirs(build)

    fp = open(npath(build, "env.sh"), "wb")
    fp.write('export BASEDIR="%s"\n' % basedir)
    fp.write('export DOWNLOADS="%s"\n' % downloads)
    fp.write('export BUILD="%s"\n' % build)
    fp.write('export PREFIX="%s"\n' % prefix)
    fp.flush()
    fp.close()

    # creating the download root
    if not os.path.exists(downloads):
        os.makedirs(downloads)

    # generating the build infrastructure subdir
    infradir = npath(build, "infra")
    if not os.path.exists(infradir):
        os.makedirs(infradir)

    fp = open(npath(infradir, "env.sh"), "wb")
    fp.write('export PATH="%s":$PATH\n' % npath(infradir, "bin"))
    for arch in [ "", "64", ]:
        fp.write('export PKG_CONFIG_PATH="%s":$PKG_CONFIG_PATH\n' % npath(infradir, "lib" + arch, "pkgconfig"))
        if platform.platform().lower().find("darwin") != -1:
            fp.write('export DYLD_FALLBACK_LIBRARY_PATH="%s":$DYLD_FALLBACK_LIBRARY_PATH\n' % npath(infradir, "lib" + arch))
        else:    
            fp.write('export LD_LIBRARY_PATH="%s":$LD_LIBRARY_PATH\n' % npath(infradir, "lib" + arch))

    # populate the scripts
    populate(build, npath(basedir, "scripts"))



if __name__ == "__main__":
    #basedir = npath(os.path.dirname(sys.argv[0]))
    #options = parse_args(sys.argv[1:])
    #configure(basedir, options.build, options.downloads, options.prefix)
    print autolib

