#!/usr/bin/env python
import sys
import os
import re
import tempfile

__version__ = "$Revision: 138 $"


class Options(object): 
    allowed_modes = dict([ ("default",   "%(platform)s-%(distro)s-%(version)s-%(arch)s"), 
                           ("full",      "%(platform)s-%(distro)s-%(version)s-%(arch)s"),
                           ("platform",  "%(platform)s",),                                
                           ("distro",    "%(distro)s",),                                
                           ("version",   "%(version)s",),                                
                           ("arch",      "%(arch)s",), ])                              
    root = "/"
    def __init__(self):
        self.platform = self.distro = self.version = self.arch = None
        self.root = self.__class__.root 
        self.outfmt = self.allowed_modes["default"]


def info(msg):
    pass


def debug(msg):
    pass


def error(msg):
    print >>sys.stderr, "*", msg
    sys.exit(1)


def npath(*args):
    "joins *args into an absolute pathname -> string"
    return os.path.normpath(os.path.abspath(os.path.expanduser(os.path.join(*args))))


def tail(filename):
    lines = open(filename, "rb").read().split("\n")
    while len(lines) != 0:
        l = lines.pop()
        if l.strip() != "": return l.strip()
    return ""


def head(filename):
    lines = open(filename, "rb").read().split("\n")
    while len(lines) != 0:
        l = lines.pop(0)
        if l.strip() != "": return l.strip()
    return ""


def awk(filename, search=None, sep=None):
    result =[]
    for line in open(filename, "rb").read().split("\n"):
        if search and search(line):
            result.append(line.split(sep))
    return result
    

def run(cmd):
    fd, name = tempfile.mkstemp()
    os.close(fd)
    fullcmd = '%s >"%s"' % (cmd, name)
    ret = os.system(fullcmd)
    data = open(name, "rb").read()
    os.unlink(name)
    return ret, data


def usage(msg=None):
    prg = os.path.basename(sys.argv[0])
    if msg:
        print "%s: %s (see -h|--help)" % (prg, msg)
        sys.exit(1)
    print """
usage: %s [options]
 reports a normalized system info string.

options:

  -h|--help               shows this message
  --version               print version for $prg
  -v|--verbose            verbose messages
  -d|--debug              debug messages
  -r|--root <dir>         use <dir> as root filesystem
                            (def: '%s')
  -m|--mode               output mode:
                            default|full   <platform>-<distro>-<version>-<arch>  
                            platform       <platform>  
                            distro         <distro>
                            version        <version>
                            arch           <arch>
  --examples              shows result string examples
  --platform <platform>   override the platform autodetection

download:
 this script can be obtained from:
  svn export https://pyvm.svn.sourceforge.net/svnroot/pyvm/projects/ident2/trunk/ident2.py

author: Antonio Cavallo, 2012-

license: copyleft
""" % (prg, Options.root)
    sys.exit(1)
    

def parse_args(args):
    global info, debug, __version__

    options = Options()

    values = []
    while len(args) != 0:
        arg = args.pop(0)
        if arg in [ "-h", "--help", ]:
            usage()
        if arg in [ "--version", ]:
            print __version__.split()[1].strip("$")
            sys.exit(0)   
        if arg in [ "-v", "--verbose", ]:
            info = lambda msg: sys.stderr.write("+ " + msg + "\n")
            continue
        if arg in [ "-d", "--debug", ]:
            info = lambda msg: sys.stderr.write("+ " + msg + "\n")
            debug = lambda msg: sys.stderr.write("d " + msg + "\n")
            continue
        if arg in [ "--platform", ]:
            options.platform = args.pop(0)
            continue
        if arg in [ "-m", "--mode", ]:
            mode = args.pop(0)
            if not mode in Options.allowed_modes.keys():
                usage("mode [%s] not allowed" % mode)
            options.outfmt = Options.allowed_modes[mode]
            continue
        if arg in [ "-f", "--format", ]:
            options.outfmt = args.pop(0)
            continue
        if arg in [ "-r", "--root", ]:
            options.root = args.pop(0)
            continue

        values.append(arg)

    if len(values) != 0:
        usage("expected no argument but found %i" % (len(values)))

    return options
            

def get_platform(root="/"):
    if root != "/":
        if os.path.exists(npath(root, "etc", "redhat-release")) or \
            os.path.exists(npath(root, "etc", "fedora-release")) or \
            os.path.exists(npath(root, "etc", "mandrake-release")) or \
            os.path.exists(npath(root, "etc", "centos-release")) or \
            os.path.exists(npath(root, "etc", "SuSE-release")) :
            info("platform detection: using root: %s" % root)
            return "linux"

    info("platform detection: runtime fallback")
    ret, uname = run("uname -a")
    debug("uname -a results in: %s" % uname.strip())
    ret, uname = run("uname -s")
    debug("uname -s results in: %s" % uname.strip())

    tag = uname.strip()
    if tag.startswith("Linux"):
        return "linux"
    if tag.startswith("OpenBSD"):
        return "openbsd"   
    if tag.startswith("FreeBSD"):
        return "freebsd"
    if tag.startswith("Darwin"):
        return "darwin"
    return "unknown"
    

def get_linux_distro(root="/"):
    if os.path.exists(npath(root, "etc", "SuSE-release")):
        result = "suse"
        tag = tail(npath(root, "etc", "SuSE-release"))
        debug("tailing SuSE-release: %s" % tag)
        if tag.startswith("PATCHLEVEL ="):
            result = "sle"
    elif os.path.exists(npath(root, "etc", "mandrake-release")):
        result = "mandrake"
    elif os.path.exists(npath(root, "etc", "fedora-release")):
        result = "fedora"
    elif os.path.exists(npath(root, "etc", "centos-release")):
        result = "centos"
    elif os.path.exists(npath(root, "etc", "redhat-release")):
        result = "redhat"

        # Old centos had no centos-release file
        tag = tail(npath(root, "etc", "redhat-release"))
        debug("tailing redhat-release: %s" % tag)
        if tag.startswith("Red Hat Enterprise Linux "):
            result = "rhel"
        elif tag.startswith("CentOS release"):
            result = "centos"
    elif os.path.exists(npath(root, "etc", "slackware-version")):
        result = "slackware"
    elif os.path.exists(npath(root, "etc", "xandros-desktop-version")):
        result = "xandros"
    else:
        result = "unknown"
    return result


def get_linux_version(root, distro):
    version = "unknown"

    if distro == "suse":
        version = awk(npath(root, "etc", "SuSE-release"), re.compile("^openSUSE").search)[0][1]
    elif distro == "sle":
        tag = tail(npath(root, "etc", "SuSE-release"))
        if tag.startswith("PATCHLEVEL ="):
            debug("heading SuSE-release (for sle): %s" % tag)
            version = head(npath(root, "etc", "SuSE-release")).split()[4]
            version += "." + tag.split("=")[1].strip()
        else:
            debug("tailing SuSE-release (for sle): %s" % tag)
            version=tag.split()[2].strip()       
    elif distro == "redhat":
        tag = tail(npath(root, "etc", "redhat-release"))
        debug("tailing redhat-release: %s" % tag)
        if tag.startswith("release "):
            version = tag.split()[1].strip()
        elif tag.startswith("Red Hat Linux release 9 (Shrike)"):
            version = "9.0"
        elif tag.startswith("Red Hat Linux release"):
            version = tag.split()[4].strip()
    elif distro == "rhel":
        tag = tail(npath(root, "etc", "redhat-release"))
        debug("tailing redhat-release: %s" % tag)
        if tag.startswith("Red Hat Enterprise Linux "):
            version = tag.split()[6].strip()
    elif distro == "centos":
        if os.path.exists(npath(root, "etc", "centos-release")):
            tag = tail(npath(root, "etc", "centos-release"))
            debug("tailing centos-release: %s" % tag)

            # eg. CentOS release 6.2 (Final)
            m0 = re.compile(r"CentOS release (?P<version>\d+([.]\d+)*) .*", re.IGNORECASE)

            if tag.startswith("CentOS Linux Advanced Server release 2.1AS"):
                version = "2.1"
            elif m0.search(tag):
                version = m0.search(tag).group("version")
            else:
                version = tag.split()[5].strip()
        else:
            tag = tail(npath(root, "etc", "redhat-release"))
            debug("tailing redhat-release (for centos): %s" % tag)
            version = tag.split()[2].strip()
    elif distro == "fedora":
        if os.path.exists(npath(root, "etc", "system-release")):
            tag = tail(npath(root, "etc", "system-release"))
            debug("tailing system-release (for fedora): %s" % tag)
            version = tag.split()[2].strip()
        else:
            tag = tail(npath(root, "etc", "fedora-release"))
            debug("tailing fedora-release: %s" % tag)
            version = tag.split()[3].strip()
    elif distro == "mandrake":
        tag = tail(npath(root, "etc", "mandrake-release"))
        debug("tailing mandrake-release: %s" % tag)
        version = tag.split()[3]
        if version == "(Community)":
            version = tag.split()[2]
    elif distro == "slackware":
        tag = tail(npath(root, "etc", "slackware-version"))
        debug("tailing slackware-version: %s" % tag)
        version = tag.split()[1]
    elif distro == "xandros":
        tag = head(npath(root, "etc", "xandros-desktop-version"))
        version = tag.split()[4]
    elif distro == "lsb":
        raise NotImplementedError("no support for lsb yet")

    return version
            

def get_linux_arch(root, distro):
    if root != "/":
        if distro == "suse":
            info("arch detection: using root: %s" % root)
            # openSUSE 11.1 (i586)
            m0 = re.compile(r"openSUSE (?P<version>\d+([.]\d+)*) [(](?P<arch>\w+)[)]", re.IGNORECASE)
            for line in open(npath(root, "etc", "SuSE-release"), "rb"):
                if m0.search(line):
                    return m0.search(line).group("arch")
            
    info("arch detection: runtime fallback")
    return run("uname -m")[1]


def detect(root="/", platform=None):
    if platform == None:
        platform = get_platform(root)
    info("platform detected '%s'" % platform)

    distro = version = arch = "unknown"

    if platform == "linux":
        distro = get_linux_distro(root)
        info("linux distro identified as: %s" % distro)
        version = get_linux_version(root, distro)
        info("linux version identified as: %s" % version)
        arch = get_linux_arch(root, distro)
        info("linux arch identified as: %s" % arch)
    else:
        if platform == "openbsd":
            distro = "openbsd"
            ret, data = run("uname -a")
            version = data.strip().split()[2]
            arch = run("uname -m")[1].strip()
        elif platform == "freebsd":
            distro = "freebsd"
            if os.path.exists(npath(ROOT, "PCBSD")):
                distro = "pcbsd"
            version = run("uname -r")[1].strip().split("-")[0]
            arch = run("uname -m")[1].strip()
        elif platform == "darwin":
            distro = "macosx"
            version = "unknown"
            for l in run("sw_vers")[1].split("\n"):
                if l.startswith("ProductVersion:"):
                    version = l.split()[1]
            arch = run("uname -m")[1].strip()
        info("distro detected: %s" % distro)
        info("version detected: %s" % version)
        info("arch detected: %s" % arch)

    return platform, distro, version, arch


def main():
    options = parse_args(sys.argv[1:])

    info("root set to '%s'" % options.root)
    info("platform set to '%s'" % options.platform)

    platform, distro, version, arch = detect(options.root, options.platform)


    info("platform  identified: %s" % platform)
    info("distro    identified: %s" % distro)
    info("version   identified: %s" % version)
    info("arch      identified: %s" % arch)

    data = { 'platform' : platform, 'distro' : distro,
              'version' : version, 'arch' : arch, }

    print options.outfmt % data


if __name__ == "__main__":
    main()
