import os
import pdb

from distutilsng.support import argparse
from distutilsng.support import logging


def npath(*args):
    "joins *args into an absolute pathname -> string"
    return os.path.normpath(os.path.abspath(os.path.expanduser(os.path.join(*args))))


def which(program, raise_=True, os_path=None):
    "returns the pathname to program -> string"
    result = program
    if not os.path.isabs(program):
        for subdir in os.environ.get('PATH', '').split(os.path.pathsep):
            pname = os.path.join(subdir, program)
            if os.path.exists(pname):
                result = pname
                break
    if os.path.exists(result):
        return os.path.normpath(result)        
    if raise_ == True:
        raise RuntimeError(u"unable to find program: %s" % program)
    elif raise_ in [ False, None, ]:
        return None
    else:
        raise raise_


class CliParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        if not 'formatter_class' in kwargs:
            kwargs['formatter_class']=argparse.ArgumentDefaultsHelpFormatter
        super(CliParser, self).__init__(*args, **kwargs)

        group = self.add_mutually_exclusive_group()
        group.add_argument("-v", "--verbose", action="store_true", default=False, help="increases logging verbosity")
        group.add_argument("-q", "--quiet", action="store_true", default=False, help="reduces logging verbosity")

        self.add_argument("--pdb", action="store_true", default=False, help="on error launch the pdb debugger")

    def parse_args(self, *args, **kwargs):
        options = super(CliParser, self).parse_args(*args, **kwargs)

        for ln in logging._levelNames:
            if isinstance(logging._levelNames[ln], str): 
                logging._levelNames[ln] = logging._levelNames[ln].lower()
        logging.basicConfig(format="%(levelname)s:%(name)s:%(func)s:%(message)s")
        logging.getLogger().setLevel(logging.INFO)
        if options.quiet == True:
            logging.getLogger().setLevel(logging.WARN)
        if options.verbose == True:
            logging.getLogger().setLevel(logging.DEBUG)

        if options.pdb == True:
            logging.warn("catching un-handled exception with pdb")
            sys.excepthook = lambda a, b, c: pdb.post_mortem(c)
        return options
